package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.json.JSONUtil;
import cn.kdan.compdf.dto.AddNewWebhookDTO;
import cn.kdan.compdf.dto.AddNewWebhookSaaSDTO;
import cn.kdan.compdf.dto.EditWebhookDTO;
import cn.kdan.compdf.entity.BackgroundEvent;
import cn.kdan.compdf.entity.BackgroundUserProject;
import cn.kdan.compdf.entity.BackgroundUserWebhook;
import cn.kdan.compdf.entity.BackgroundWebhookEvent;
import cn.kdan.compdf.enums.RabbitMQEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.mapper.BackgroundUserWebhookMapper;
import cn.kdan.compdf.service.BackgroundEventService;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.service.BackgroundUserWebhookService;
import cn.kdan.compdf.service.BackgroundWebhookEventService;
import cn.kdan.compdf.utils.MD5Util;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.BackgroundEventVO;
import cn.kdan.compdf.vo.BackgroundUserWebhookVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.assertj.core.util.Lists;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@Service
public class BackgroundUserWebhookServiceImpl extends ServiceImpl<BackgroundUserWebhookMapper, BackgroundUserWebhook> implements BackgroundUserWebhookService {

    @Autowired
    private BackgroundEventService backgroundEventService;
    @Autowired
    private BackgroundWebhookEventService backgroundWebhookEventService;
    @Autowired
    private BackgroundUserProjectService backgroundUserProjectService;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public IPage<BackgroundUserWebhook> page(BackgroundUserWebhook query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    /**
     * 获取Webhooks列表
     * @param userId 用户id
     * @return 用户Webhooks列表
     */
    @Override
    public List<BackgroundUserWebhookVO> getWebhookList(Long userId) {
        // 查找用户下面状态正常的Webhook记录
        LambdaQueryWrapper<BackgroundUserWebhook> backgroundUserWebhookeq = new LambdaQueryWrapper<BackgroundUserWebhook>()
                .between(BackgroundUserWebhook::getStatus, 1, 2)
                .eq(BackgroundUserWebhook::getUserId, TokenUtil.getRequestHeader().getId())
                .orderByDesc(BackgroundUserWebhook::getCreateDate);
        List<BackgroundUserWebhook> backgroundUserWebhookList = this.baseMapper.selectList(backgroundUserWebhookeq);
        if (CollectionUtil.isNotEmpty(backgroundUserWebhookList)) {
            List<BackgroundUserWebhookVO> backgroundUserWebhookVOList = Lists.newArrayList();
            // 获取系统所有事件
            List<BackgroundEvent> backgroundEventList = backgroundEventService.getEventList();
            backgroundUserWebhookList.forEach(c -> {
                BackgroundUserWebhookVO backgroundUserWebhookVO = new BackgroundUserWebhookVO();
                BeanUtil.copyProperties(c, backgroundUserWebhookVO);
                // 获取webhook关联事件
                LambdaQueryWrapper<BackgroundWebhookEvent> backgroundWebhookEventeq = new LambdaQueryWrapper<BackgroundWebhookEvent>()
                        .eq(BackgroundWebhookEvent::getStatus, 1)
                        .eq(BackgroundWebhookEvent::getWebhookId, c.getId());
                List<BackgroundWebhookEvent> backgroundWebhookEventList = backgroundWebhookEventService.list(backgroundWebhookEventeq);
                if (CollectionUtil.isNotEmpty(backgroundWebhookEventList)) {
                    List<String> events = Lists.newArrayList();
                    for (BackgroundWebhookEvent backgroundWebhookEvent : backgroundWebhookEventList) {
                        // 查找返回事件名
                        backgroundEventList.stream().filter(d -> d.getId().equals(backgroundWebhookEvent.getEventId())).findFirst().ifPresent(backgroundEvent -> events.add(backgroundEvent.getEventName()));
                    }
                    backgroundUserWebhookVO.setEvents(events);
                }
                backgroundUserWebhookVOList.add(backgroundUserWebhookVO);
            });
            return backgroundUserWebhookVOList;
        } else {
            return Collections.emptyList();
        }
    }

    /**
     * 获取系统所有事件
     * @return 系统所有事件
     */
    @Override
    public List<BackgroundEventVO> getEventList() {
        List<BackgroundEvent> eventList = backgroundEventService.getEventList();
        List<BackgroundEventVO> backgroundEventVOList = Lists.newArrayList();
        for (BackgroundEvent backgroundEvent : eventList) {
            BackgroundEventVO backgroundEventVO = new BackgroundEventVO();
            BeanUtil.copyProperties(backgroundEvent, backgroundEventVO);
            backgroundEventVOList.add(backgroundEventVO);
        }
        return backgroundEventVOList;
    }

    @Override
    public List<Long> getEnableEventIdList(Long webhookId) {
        LambdaQueryWrapper<BackgroundWebhookEvent> eq = new LambdaQueryWrapper<BackgroundWebhookEvent>()
                .eq(BackgroundWebhookEvent::getStatus, 1)
                .eq(BackgroundWebhookEvent::getWebhookId, webhookId);
        List<BackgroundWebhookEvent> list = backgroundWebhookEventService.list(eq);
        if (CollectionUtil.isNotEmpty(list)) {
            return list.stream().map(BackgroundWebhookEvent::getEventId).collect(Collectors.toList());
        } else {
            return Collections.emptyList();
        }
    }

    /**
     * 添加新Webhook
     * @param addNewWebhookDTO 请求
     */
    @Override
    public void addNewWebhook(AddNewWebhookDTO addNewWebhookDTO) {
        // 参数校验
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .eq(BackgroundUserProject::getUserId, addNewWebhookDTO.getUserId())
                .eq(BackgroundUserProject::getId, addNewWebhookDTO.getProjectId());
        BackgroundUserProject backgroundUserProject = backgroundUserProjectService.getOne(eq);
        if (null == backgroundUserProject) {
            log.error("新增webhook项目id错误：" + addNewWebhookDTO.getProjectId());
            throw new BusinessException("parameter error");
        }

        BackgroundUserWebhook backgroundUserWebhook = new BackgroundUserWebhook();
        BeanUtil.copyProperties(addNewWebhookDTO, backgroundUserWebhook);
        // 设置回调token
        backgroundUserWebhook.setSecretToken(MD5Util.MD5Encode(UUID.randomUUID().toString()));
        backgroundUserWebhook.setStatus(1);
        this.baseMapper.insert(backgroundUserWebhook);
        // webhook关联事件
//        List<BackgroundWebhookEvent> backgroundWebhookEventList = Lists.newArrayList(); // 批量插入
        addNewWebhookDTO.getEventsId().forEach(c -> {
            BackgroundWebhookEvent backgroundWebhookEvent = new BackgroundWebhookEvent();
            backgroundWebhookEvent.setWebhookId(backgroundUserWebhook.getId());
            backgroundWebhookEvent.setEventId(c);
            backgroundWebhookEvent.setStatus(1);
            backgroundWebhookEventService.save(backgroundWebhookEvent);
        });
        // 发送消息同步SaaS服务
        AddNewWebhookSaaSDTO addNewWebhookSaaSDTO = new AddNewWebhookSaaSDTO();
        BeanUtil.copyProperties(addNewWebhookDTO, addNewWebhookSaaSDTO);
        addNewWebhookSaaSDTO.setWebhookId(backgroundUserWebhook.getId());
        addNewWebhookSaaSDTO.setSecretToken(backgroundUserWebhook.getSecretToken());
        String jsonStr = JSONUtil.toJsonStr(addNewWebhookSaaSDTO);
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_WEBHOOKS_ADD_ROUTING_KEY, jsonStr);
    }

    @Override
    public void editWebhook(EditWebhookDTO editWebhookDTO) {
        // 参数校验
        LambdaQueryWrapper<BackgroundUserProject> eq = new LambdaQueryWrapper<BackgroundUserProject>()
                .eq(BackgroundUserProject::getUserId, editWebhookDTO.getUserId())
                .eq(BackgroundUserProject::getId, editWebhookDTO.getProjectId());
        BackgroundUserProject backgroundUserProject = backgroundUserProjectService.getOne(eq);
        if (null == backgroundUserProject) {
            log.error("编辑webhook项目id错误：" + editWebhookDTO.getProjectId());
            throw new BusinessException("parameter error");
        }

        BackgroundUserWebhook backgroundUserWebhook = new BackgroundUserWebhook();
        BeanUtil.copyProperties(editWebhookDTO, backgroundUserWebhook);
        this.baseMapper.updateById(backgroundUserWebhook);
        // 清除原有关联事件
        backgroundWebhookEventService.cleanWebhookEventByWebhookId(editWebhookDTO.getId());
        // 新增webhook关联事件
        editWebhookDTO.getEventsId().forEach(c -> {
            BackgroundWebhookEvent backgroundWebhookEvent = new BackgroundWebhookEvent();
            backgroundWebhookEvent.setWebhookId(editWebhookDTO.getId());
            backgroundWebhookEvent.setEventId(c);
            backgroundWebhookEvent.setStatus(1);
            backgroundWebhookEventService.save(backgroundWebhookEvent);
        });
        // 发送消息同步SaaS服务
        String jsonStr = JSONUtil.toJsonStr(editWebhookDTO);
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_WEBHOOKS_UPD_ROUTING_KEY, jsonStr);
    }

    @Override
    public void deleteById(Long id) {
        BackgroundUserWebhook backgroundUserWebhook = new BackgroundUserWebhook();
        backgroundUserWebhook.setId(id);
        backgroundUserWebhook.setStatus(0);
        this.baseMapper.updateById(backgroundUserWebhook);
        // 清除webhook关联事件
        backgroundWebhookEventService.cleanWebhookEventByWebhookId(id);
        // 发送消息同步SaaS服务
        rabbitTemplate.convertAndSend(RabbitMQEnum.BACKGROUND_EXCHANGE, RabbitMQEnum.BACKGROUND_WEBHOOKS_DEL_ROUTING_KEY, id);
    }

}
